#!/bin/bash

mkittool__popd() {
    #
    # popd from $1, warn and false if it goes wrong
    #
    local path=$1
    test -n "$path" || { warn "usage: mkittool__popd PATH"; return 2; }
    debug "popding from: $path"
    popd >/dev/null && return 0
    warn "could not popd from: $path"
    return 3
}

mkittool__pushd() {
    #
    # pushd to $1, warn and false if it goes wrong
    #
    local path=$1
    test -n "$path" || { warn "usage: mkittool__pushd PATH"; return 2; }
    debug "pushding to: $path"
    pushd "$path" >/dev/null && return 0
    warn "could not pushd to: $path"
    return 3
}

mkittool__remind_head() {
    #
    # Remind user what is the latest commit
    #
    local path=$1
    mkittool__pushd "$path" || die
        "$_MKITTOOL_THINK" "latest commit:"
        "$_MKITTOOL_THINK" " : hash $(git log -1 --format='%h')"
        "$_MKITTOOL_THINK" " : subj $(git log -1 --format='%s')"
        "$_MKITTOOL_THINK" " ' time $(git log -1 --format='%ai')"
    mkittool__popd "$path" || die
}

mkittool__run_cmd() {
    #
    # Show command $@ including env pairs and run it
    #
    # Usage:
    #     mkittool__run_cmd [VAR=VAL].. [--] CMD [ARG]..
    #
    # Run CMD with ARGS and any envvars set up, announcing it first
    # if verbose mode is enabled.
    #
    # If verbose is enabled, command is first printed in a way that
    # makes it clear which part is environment variables (VAR=VAL)
    # and which is the command itself (CMD and possible ARGs), while
    # still producing a valid and informative Bash code snippet which
    # can be shared when asking for help.
    #
    # For example, this command:
    #
    #     mkittool__run_cmd "FOO=bar" "BAZ=123" foo bar "baz with space"
    #
    # will print:
    #
    #    running foo:
    #        FOO="bar" \
    #        BAZ="123" \
    #            foo bar "baz with space"
    #
    # to stdout (using Shellfu's pretty; see 'sfodo pretty') before
    # running the command.
    #
    # Exit status of this function is exit status of the command run,
    # except for when parsing fails, in which case the status is 2.
    # (Note that parsing can only fail if no CMD is provided.)
    #
    # The first argument that does not contain equals sign is considered
    # CMD and rest of them is considered ARG.  To enforce treatment of
    # an argument as CMD even if it contains equals sign, include `--`
    # before it; this delimiter argument will be skipped.  Only up to
    # one appearance of `--` is treated specially by this function, in
    # particular before the CMD, i.e. it's safe for CMD or ARG to use
    # it.
    #
    # Printed code is quoted using printf builtin (`%q` special Bash
    # format), so it might not be always most readable but it should
    # be safe to run separately.
    #
    # All printed lines are prefixed by four spaces to make it easy to
    # prepend with progress message, e.g.:
    #
    #     think "running debuild:
    #     mkittool__think_cmd "DEBUILD_OPTIONS=-foo -bar" debuild -us -uc
    #
    local arg       # currently parsed argument
    local state=e   # parser state (e=envvar, c=cmd/arg)
    local env=()    # parsed env pairs
    local cmd=()    # parsed CMD and any ARGs
    local lines=()  # info lines
    local pair      # envvar pair when formatting
    local name      # ^ name
    local value     # ^ value
    for arg in "$@"; do case "$state:$arg" in
        e:--)   state=c; ;;
        e:*=*)  state=e; env+=("$arg") ;;
        *)      state=c; cmd+=("$arg") ;;
    esac done
    test "${#cmd[*]}" -gt 0 || { warn "no CMD?"; return 2; }
    lines=("running ${cmd[0]}:")
    for pair in "${env[@]}"; do
        name=${pair%%=*}; value=${pair#"$name="}
        lines+=("    $name=$(printf %q "$value") \\")
    done
    lines+=("       $(printf ' %q' "${cmd[@]}")")
    think -l "${lines[@]}"
    env "${env[@]}" "${cmd[@]}"
}

#
# Which command to think with
#
_MKITTOOL_THINK=${_MKITTOOL_THINK:-think}

#shellfu module-version=0.0.18
